import { z } from 'zod';
import { MastraEngine } from '../engine';
import { Logger } from '../logger';
import { Telemetry } from '../telemetry';
import { Step } from './step';
import { WorkflowLogMessage, StepConfig, StepResult, RetryConfig, StepGraph } from './types';
export declare class Workflow<TSteps extends Step<any, any, any>[] = any, TTriggerSchema extends z.ZodType<any> = any> {
    #private;
    name: string;
    /**
     * Creates a new Workflow instance
     * @param name - Identifier for the workflow (not necessarily unique)
     * @param logger - Optional logger instance
     */
    constructor({ name, logger, engine, triggerSchema, retryConfig, telemetry, }: {
        name: string;
        logger?: Logger<WorkflowLogMessage>;
        engine?: MastraEngine;
        triggerSchema?: TTriggerSchema;
        retryConfig?: RetryConfig;
        telemetry?: Telemetry;
    });
    /**
     * Initializes the XState machine for the workflow
     *
     * Registers the machine's types, actions, actors, initial context, entry actions, initial state, and states
     * @returns The initialized machine
     */
    private initializeMachine;
    step(step: Step<any, any, any>, config?: StepConfig<any, any, any>): this;
    then<TStep extends Step<any, any, any>, CondStep extends Step<any, any, any>, VarStep extends Step<any, any, any>>(step: TStep, config?: StepConfig<TStep, CondStep, VarStep>): this;
    /**
     * Executes the workflow with the given trigger data
     * @param triggerData - Initial data to start the workflow with
     * @returns Promise resolving to workflow results or rejecting with error
     * @throws Error if trigger schema validation fails
     */
    execute({ triggerData, loadSnapshot, }?: {
        triggerData?: z.infer<TTriggerSchema>;
        loadSnapshot?: {
            runId: string;
        };
    }): Promise<{
        triggerData?: z.infer<TTriggerSchema>;
        results: Record<string, StepResult<any>>;
        runId: string;
    }>;
    /**
     * Rebuilds the machine with the current steps configuration and validates the workflow
     *
     * This is the last step of a workflow builder method chain
     * @throws Error if validation fails
     *
     * @returns this instance for method chaining
     */
    commit(): this;
    __registerEngine(engine?: MastraEngine): void;
    __registerLogger(logger?: Logger<WorkflowLogMessage>): void;
    __registerTelemetry(telemetry?: Telemetry): void;
    get stepGraph(): StepGraph;
}
